{-# LANGUAGE MultiParamTypeClasses, FunctionalDependencies #-}
{-# OPTIONS_GHC -fno-warn-duplicate-exports #-}
module Data.FpML.V53.Eqd
  ( module Data.FpML.V53.Eqd
  , module Data.FpML.V53.Shared.EQ
  ) where
 
import Text.XML.HaXml.Schema.Schema (SchemaType(..),SimpleType(..),Extension(..),Restricts(..))
import Text.XML.HaXml.Schema.Schema as Schema
import qualified Text.XML.HaXml.Schema.PrimitiveTypes as Xsd
import {-# SOURCE #-} Data.FpML.V53.Shared.EQ
 
-- | A type for defining the broker equity options. 
data BrokerEquityOption
instance Eq BrokerEquityOption
instance Show BrokerEquityOption
instance SchemaType BrokerEquityOption
instance Extension BrokerEquityOption EquityDerivativeShortFormBase
instance Extension BrokerEquityOption EquityDerivativeBase
instance Extension BrokerEquityOption Product
 
-- | A type for defining exercise procedures associated with an 
--   American style exercise of an equity option. This entity 
--   inherits from the type SharedAmericanExercise. 
data EquityAmericanExercise
instance Eq EquityAmericanExercise
instance Show EquityAmericanExercise
instance SchemaType EquityAmericanExercise
instance Extension EquityAmericanExercise SharedAmericanExercise
instance Extension EquityAmericanExercise Exercise
 
-- | A type for defining exercise procedures associated with a 
--   Bermuda style exercise of an equity option. The term 
--   Bermuda is adopted in FpML for consistency with the ISDA 
--   Definitions. 
data EquityBermudaExercise
instance Eq EquityBermudaExercise
instance Show EquityBermudaExercise
instance SchemaType EquityBermudaExercise
instance Extension EquityBermudaExercise SharedAmericanExercise
instance Extension EquityBermudaExercise Exercise
 
-- | A type for defining the common features of equity 
--   derivatives. 
data EquityDerivativeBase
instance Eq EquityDerivativeBase
instance Show EquityDerivativeBase
instance SchemaType EquityDerivativeBase
instance Extension EquityDerivativeBase Product
 
-- | type for defining the common features of equity 
--   derivatives. 
data EquityDerivativeLongFormBase
instance Eq EquityDerivativeLongFormBase
instance Show EquityDerivativeLongFormBase
instance SchemaType EquityDerivativeLongFormBase
instance Extension EquityDerivativeLongFormBase EquityDerivativeBase
 
-- | A type for defining short form equity option basic 
--   features. 
data EquityDerivativeShortFormBase
instance Eq EquityDerivativeShortFormBase
instance Show EquityDerivativeShortFormBase
instance SchemaType EquityDerivativeShortFormBase
instance Extension EquityDerivativeShortFormBase EquityDerivativeBase
 
-- | A type for defining exercise procedures associated with a 
--   European style exercise of an equity option. 
data EquityEuropeanExercise
instance Eq EquityEuropeanExercise
instance Show EquityEuropeanExercise
instance SchemaType EquityEuropeanExercise
instance Extension EquityEuropeanExercise Exercise
 
-- | A type for defining exercise procedures for equity options. 
data EquityExerciseValuationSettlement
instance Eq EquityExerciseValuationSettlement
instance Show EquityExerciseValuationSettlement
instance SchemaType EquityExerciseValuationSettlement
 
-- | A type for defining equity forwards. 
data EquityForward
instance Eq EquityForward
instance Show EquityForward
instance SchemaType EquityForward
instance Extension EquityForward EquityDerivativeLongFormBase
instance Extension EquityForward EquityDerivativeBase
instance Extension EquityForward Product
 
-- | A type for defining the multiple exercise provisions of an 
--   American or Bermuda style equity option. 
data EquityMultipleExercise
instance Eq EquityMultipleExercise
instance Show EquityMultipleExercise
instance SchemaType EquityMultipleExercise
 
-- | A type for defining equity options. 
data EquityOption
instance Eq EquityOption
instance Show EquityOption
instance SchemaType EquityOption
instance Extension EquityOption EquityDerivativeLongFormBase
instance Extension EquityOption EquityDerivativeBase
instance Extension EquityOption Product
 
-- | A type for defining Equity Option Termination. 
data EquityOptionTermination
instance Eq EquityOptionTermination
instance Show EquityOptionTermination
instance SchemaType EquityOptionTermination
 
-- | A type for defining equity option transaction supplements. 
data EquityOptionTransactionSupplement
instance Eq EquityOptionTransactionSupplement
instance Show EquityOptionTransactionSupplement
instance SchemaType EquityOptionTransactionSupplement
instance Extension EquityOptionTransactionSupplement EquityDerivativeShortFormBase
instance Extension EquityOptionTransactionSupplement EquityDerivativeBase
instance Extension EquityOptionTransactionSupplement Product
 
-- | A type for defining PrePayment. 
data PrePayment
instance Eq PrePayment
instance Show PrePayment
instance SchemaType PrePayment
instance Extension PrePayment PaymentBase
 
-- | A component describing a Broker View of an Equity Option. 
elementBrokerEquityOption :: XMLParser BrokerEquityOption
elementToXMLBrokerEquityOption :: BrokerEquityOption -> [Content ()]
 
-- | A component describing an Equity Forward product. 
elementEquityForward :: XMLParser EquityForward
elementToXMLEquityForward :: EquityForward -> [Content ()]
 
-- | A component describing an Equity Option product. 
elementEquityOption :: XMLParser EquityOption
elementToXMLEquityOption :: EquityOption -> [Content ()]
 
-- | A component describing an Equity Option Transaction 
--   Supplement. 
elementEquityOptionTransactionSupplement :: XMLParser EquityOptionTransactionSupplement
elementToXMLEquityOptionTransactionSupplement :: EquityOptionTransactionSupplement -> [Content ()]
 
