{-# LANGUAGE MultiParamTypeClasses, FunctionalDependencies #-}
{-# OPTIONS_GHC -fno-warn-duplicate-exports #-}
module Data.FpML.V53.Shared.Option
  ( module Data.FpML.V53.Shared.Option
  , module Data.FpML.V53.Asset
  ) where
 
import Text.XML.HaXml.Schema.Schema (SchemaType(..),SimpleType(..),Extension(..),Restricts(..))
import Text.XML.HaXml.Schema.Schema as Schema
import qualified Text.XML.HaXml.Schema.PrimitiveTypes as Xsd
import {-# SOURCE #-} Data.FpML.V53.Asset
 
-- | As per ISDA 2002 Definitions. 
data Asian
instance Eq Asian
instance Show Asian
instance SchemaType Asian
 
-- | An un ordered list of weighted averaging observations. 
data AveragingObservationList
instance Eq AveragingObservationList
instance Show AveragingObservationList
instance SchemaType AveragingObservationList
 
-- | Period over which an average value is taken. 
data AveragingPeriod
instance Eq AveragingPeriod
instance Show AveragingPeriod
instance SchemaType AveragingPeriod
 
-- | Method of generating a series of dates. 
data AveragingSchedule
instance Eq AveragingSchedule
instance Show AveragingSchedule
instance SchemaType AveragingSchedule
 
-- | As per ISDA 2002 Definitions. 
data Barrier
instance Eq Barrier
instance Show Barrier
instance SchemaType Barrier
 
-- | A type for defining a calendar spread feature. 
data CalendarSpread
instance Eq CalendarSpread
instance Show CalendarSpread
instance SchemaType CalendarSpread
 
-- | A classified non negative payment. 
data ClassifiedPayment
instance Eq ClassifiedPayment
instance Show ClassifiedPayment
instance SchemaType ClassifiedPayment
instance Extension ClassifiedPayment NonNegativePayment
instance Extension ClassifiedPayment PaymentBaseExtended
instance Extension ClassifiedPayment PaymentBase
 
-- | Specifies the conditions to be applied for converting into 
--   a reference currency when the actual currency rate is not 
--   determined upfront. 
data Composite
instance Eq Composite
instance Show Composite
instance SchemaType Composite
 
data CreditEventNotice
instance Eq CreditEventNotice
instance Show CreditEventNotice
instance SchemaType CreditEventNotice
 
data CreditEvents
instance Eq CreditEvents
instance Show CreditEvents
instance SchemaType CreditEvents
 
-- | Reference to credit events. 
data CreditEventsReference
instance Eq CreditEventsReference
instance Show CreditEventsReference
instance SchemaType CreditEventsReference
instance Extension CreditEventsReference Reference
 
data FailureToPay
instance Eq FailureToPay
instance Show FailureToPay
instance SchemaType FailureToPay
 
-- | Payment made following trigger occurence. 
data FeaturePayment
instance Eq FeaturePayment
instance Show FeaturePayment
instance SchemaType FeaturePayment
instance Extension FeaturePayment PaymentBase
 
-- | Frequency Type. 
data FrequencyType
data FrequencyTypeAttributes
instance Eq FrequencyType
instance Eq FrequencyTypeAttributes
instance Show FrequencyType
instance Show FrequencyTypeAttributes
instance SchemaType FrequencyType
instance Extension FrequencyType Scheme
 
-- | A type for defining Fx Features. 
data FxFeature
instance Eq FxFeature
instance Show FxFeature
instance SchemaType FxFeature
 
data GracePeriodExtension
instance Eq GracePeriodExtension
instance Show GracePeriodExtension
instance SchemaType GracePeriodExtension
 
-- | Knock In means option to exercise comes into existence. 
--   Knock Out means option to exercise goes out of existence. 
data Knock
instance Eq Knock
instance Show Knock
instance SchemaType Knock
 
-- | Defines the handling of an averaging date market disruption 
--   for an equity derivative transaction. 
data MarketDisruption
data MarketDisruptionAttributes
instance Eq MarketDisruption
instance Eq MarketDisruptionAttributes
instance Show MarketDisruption
instance Show MarketDisruptionAttributes
instance SchemaType MarketDisruption
instance Extension MarketDisruption Scheme
 
data NotifyingParty
instance Eq NotifyingParty
instance Show NotifyingParty
instance SchemaType NotifyingParty
 
-- | A type for defining the common features of options. 
data Option
instance Eq Option
instance Show Option
instance SchemaType Option
instance Extension Option Product
 
-- | A type for defining the common features of options. 
data OptionBase
instance Eq OptionBase
instance Show OptionBase
instance SchemaType OptionBase
instance Extension OptionBase Option
 
-- | Base type for options starting with the 4-3 release, until 
--   we refactor the schema as part of the 5-0 release series. 
data OptionBaseExtended
instance Eq OptionBaseExtended
instance Show OptionBaseExtended
instance SchemaType OptionBaseExtended
instance Extension OptionBaseExtended OptionBase
 
-- | A type for defining option features. 
data OptionFeature
instance Eq OptionFeature
instance Show OptionFeature
instance SchemaType OptionFeature
 
-- | A type for defining the strike price for an option as a 
--   numeric value without currency. 
data OptionNumericStrike
instance Eq OptionNumericStrike
instance Show OptionNumericStrike
instance SchemaType OptionNumericStrike
 
-- | A type for defining the strike price for an equity option. 
--   The strike price is either: (i) in respect of an index 
--   option transaction, the level of the relevant index 
--   specified or otherwise determined in the transaction; or 
--   (ii) in respect of a share option transaction, the price 
--   per share specified or otherwise determined in the 
--   transaction. This can be expressed either as a percentage 
--   of notional amount or as an absolute value. 
data OptionStrike
instance Eq OptionStrike
instance Show OptionStrike
instance SchemaType OptionStrike
instance Extension OptionStrike OptionNumericStrike
 
-- | Type which contains pass through payments. 
data PassThrough
instance Eq PassThrough
instance Show PassThrough
instance SchemaType PassThrough
 
-- | Type to represent a single pass through payment. 
data PassThroughItem
instance Eq PassThroughItem
instance Show PassThroughItem
instance SchemaType PassThroughItem
 
-- | A type for defining a premium. 
data Premium
instance Eq Premium
instance Show Premium
instance SchemaType Premium
instance Extension Premium SimplePayment
instance Extension Premium PaymentBase
 
data PubliclyAvailableInformation
instance Eq PubliclyAvailableInformation
instance Show PubliclyAvailableInformation
instance SchemaType PubliclyAvailableInformation
 
-- | Determines the currency rate that the seller of the equity 
--   amounts will apply at each valuation date for converting 
--   the respective amounts into a currency that is different 
--   from the currency denomination of the underlyer. 
data Quanto
instance Eq Quanto
instance Show Quanto
instance SchemaType Quanto
 
data Restructuring
instance Eq Restructuring
instance Show Restructuring
instance SchemaType Restructuring
 
data RestructuringType
data RestructuringTypeAttributes
instance Eq RestructuringType
instance Eq RestructuringTypeAttributes
instance Show RestructuringType
instance Show RestructuringTypeAttributes
instance SchemaType RestructuringType
instance Extension RestructuringType Scheme
 
-- | A type for definining equity option simple strike or 
--   calendar spread strategy features. 
data StrategyFeature
instance Eq StrategyFeature
instance Show StrategyFeature
instance SchemaType StrategyFeature
 
-- | A type for defining a strike spread feature. 
data StrikeSpread
instance Eq StrikeSpread
instance Show StrikeSpread
instance SchemaType StrikeSpread
 
-- | Trigger point at which feature is effective. 
data Trigger
instance Eq Trigger
instance Show Trigger
instance SchemaType Trigger
 
-- | Observation point for trigger. 
data TriggerEvent
instance Eq TriggerEvent
instance Show TriggerEvent
instance SchemaType TriggerEvent
 
-- | A single weighted averaging observation. 
data WeightedAveragingObservation
instance Eq WeightedAveragingObservation
instance Show WeightedAveragingObservation
instance SchemaType WeightedAveragingObservation
 
 
 
 
