{-# LANGUAGE MultiParamTypeClasses, FunctionalDependencies #-}
{-# OPTIONS_GHC -fno-warn-duplicate-exports #-}
module Data.FpML.V53.Valuation
  ( module Data.FpML.V53.Valuation
  , module Data.FpML.V53.Enum
  , module Data.FpML.V53.Riskdef
  , module Data.FpML.V53.Msg
  , module Data.FpML.V53.Events.Business
  ) where
 
import Text.XML.HaXml.Schema.Schema (SchemaType(..),SimpleType(..),Extension(..),Restricts(..))
import Text.XML.HaXml.Schema.Schema as Schema
import qualified Text.XML.HaXml.Schema.PrimitiveTypes as Xsd
import {-# SOURCE #-} Data.FpML.V53.Enum
import {-# SOURCE #-} Data.FpML.V53.Riskdef
import {-# SOURCE #-} Data.FpML.V53.Msg
import {-# SOURCE #-} Data.FpML.V53.Events.Business
 
-- | A structure that holds a set of measures about an asset, 
--   including possibly their sensitivities. 
data AssetValuation
instance Eq AssetValuation
instance Show AssetValuation
instance SchemaType AssetValuation
instance Extension AssetValuation Valuation
 
-- | A valuation scenario that is derived from another valuation 
--   scenario. 
data DerivedValuationScenario
instance Eq DerivedValuationScenario
instance Show DerivedValuationScenario
instance SchemaType DerivedValuationScenario
 
-- | Some kind of numerical measure about an asset, eg. its NPV, 
--   together with characteristics of that measure, together 
--   with optional sensitivities. 
data Quotation
instance Eq Quotation
instance Show Quotation
instance SchemaType Quotation
 
-- | The roles of the parties in reporting information such as 
--   positions. 
data ReportingRoles
instance Eq ReportingRoles
instance Show ReportingRoles
instance SchemaType ReportingRoles
 
-- | An servicing date relevant for a trade structure, such as a 
--   payment or a reset. 
data ScheduledDate
instance Eq ScheduledDate
instance Show ScheduledDate
instance SchemaType ScheduledDate
 
-- | A list of dates (cash flows, resets, etc.) that are 
--   relevant for this structure, e.g. next cash flow, last 
--   reset, etc. Provides a way to list upcoming or recent 
--   servicing dates related to this trade stream in a way that 
--   is simpler and more flexible than the FpML "cashflows" 
--   structure. 
data ScheduledDates
instance Eq ScheduledDates
instance Show ScheduledDates
instance SchemaType ScheduledDates
 
-- | A scheme used to identify the type of a stream scheduled 
--   servicing date. 
data ScheduledDateType
data ScheduledDateTypeAttributes
instance Eq ScheduledDateType
instance Eq ScheduledDateTypeAttributes
instance Show ScheduledDateType
instance Show ScheduledDateTypeAttributes
instance SchemaType ScheduledDateType
instance Extension ScheduledDateType Scheme
 
-- | The sensitivity of a value to a defined change in input 
--   parameters. 
data Sensitivity
data SensitivityAttributes
instance Eq Sensitivity
instance Eq SensitivityAttributes
instance Show Sensitivity
instance Show SensitivityAttributes
instance SchemaType Sensitivity
instance Extension Sensitivity Xsd.Decimal
 
-- | A collection of sensitivities. References a definition that 
--   explains the meaning/type of the sensitivities. 
data SensitivitySet
instance Eq SensitivitySet
instance Show SensitivitySet
instance SchemaType SensitivitySet
 
-- | A set of valuation. 
data Valuations
instance Eq Valuations
instance Show Valuations
instance SchemaType Valuations
 
-- | A set of valuation inputs and results. This structure can 
--   be used for requesting valuations, or for reporting them. 
--   In general, the request fills in fewer elements. 
data ValuationSet
instance Eq ValuationSet
instance Show ValuationSet
instance SchemaType ValuationSet
 
-- | The amount of detail provided in the valuation set, e.g. is 
--   market environment data provided, are risk definitions 
--   provided, etc. 
data ValuationSetDetail
data ValuationSetDetailAttributes
instance Eq ValuationSetDetail
instance Eq ValuationSetDetailAttributes
instance Show ValuationSetDetail
instance Show ValuationSetDetailAttributes
instance SchemaType ValuationSetDetail
instance Extension ValuationSetDetail Scheme
 
elementValuationSet :: XMLParser ValuationSet
elementToXMLValuationSet :: ValuationSet -> [Content ()]
 
 
 
 
-- | A collection of related trades or positions and the 
--   corresponding aggregate exposures generated by these. 
data Position
instance Eq Position
instance Show Position
instance SchemaType Position
 
 
-- | A list of events that have affected a position. 
data PositionHistory
instance Eq PositionHistory
instance Show PositionHistory
instance SchemaType PositionHistory
 
-- | The items (trades, trade references, holdings, other 
--   positions) that comprise this position. Currently a 
--   position may consist only of a single trade, a reference to 
--   a previously submitted position, or a reference to the 
--   trade. The choice structure is optional to allow extensions 
--   to be placed within this container. 
data PositionConstituent
instance Eq PositionConstituent
instance Show PositionConstituent
instance SchemaType PositionConstituent
